<?php

/**
 * @file
 * Contains \Drupal\yamlform\Entity\YamlFormSubmissionListBuilder.
 */

namespace Drupal\yamlform;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Database\Query\AlterableInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;

/**
 * Provides a list controller for yamlform submission entity.
 *
 * @ingroup yamlform
 */
class YamlFormSubmissionListBuilder extends EntityListBuilder {

  /**
   * Submission state starred.
   */
  const STATE_STARRED = 'starred';

  /**
   * Submission state unstarred.
   */
  const STATE_UNSTARRED = 'unstarred';

  /**
   * The YAML form.
   *
   * @var \Drupal\yamlform\YamlFormInterface
   */
  protected $yamlform;

  /**
   * The entity that a YAML form is attached to. Currently only applies to nodes.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $sourceEntity;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * The columns being displayed.
   *
   * @var array
   */
  protected $columns = [];

  /**
   * The YAML form elements.
   *
   * @var array
   */
  protected $elements = [];

  /**
   * Search keys.
   *
   * @var string
   */
  protected $keys;

  /**
   * Search state.
   *
   * @var string
   */
  protected $state;

  /**
   * The YAMl form element manager.
   *
   * @var \Drupal\yamlform\YamlFormElementManagerInterface
   */
  protected $elementManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
    parent::__construct($entity_type, $storage);

    /** @var \Drupal\yamlform\YamlFormRequestInterface $yamlform_request */
    $yamlform_request = \Drupal::service('yamlform.request');

    $this->keys = \Drupal::request()->query->get('search');
    $this->state = \Drupal::request()->query->get('state');

    list($this->yamlform, $this->sourceEntity) = $yamlform_request->getYamlFormEntities();
    $base_route_name = $this->getBaseRouteName();

    $this->account = (\Drupal::routeMatch()->getRouteName() == "$base_route_name.yamlform.submissions") ? \Drupal::currentUser() : NULL;

    $this->elementManager = \Drupal::service('plugin.manager.yamlform.element');

    if (\Drupal::routeMatch()->getRouteName() == "$base_route_name.yamlform.results_table") {
      $this->elements = $this->yamlform->getElementsFlattenedAndHasValue();
      // Use the default format when displaying each element.
      foreach ($this->elements as &$element) {
        unset($element['#format']);
      }
      $this->columns = $this->getStorage()->getCustomColumns($this->yamlform, $this->sourceEntity, $this->account, TRUE);
    }
    else {
      $this->columns = $this->getStorage()->getDefaultColumns($this->yamlform, $this->sourceEntity, $this->account, FALSE);
    }

    $this->limit = $this->getStorage()->getCustomLimit($this->yamlform, $this->sourceEntity);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    if ($this->yamlform) {
      if ($this->account) {
        $build['#title'] = $this->t('Submissions to %yamlform for %user', [
          '%yamlform' => $this->yamlform->label(),
          '%user' => $this->account->getDisplayName(),
        ]);
      }
    }

    // Add the filter.
    $state_options = [
      '' => $this->t('All [@total]', ['@total' => $this->getTotal(NULL, NULL)]),
      'starred' => $this->t('Starred [@total]', ['@total' => $this->getTotal(NULL, self::STATE_STARRED)]),
      'unstarred' => $this->t('Unstarred [@total]', ['@total' => $this->getTotal(NULL, self::STATE_UNSTARRED)]),
    ];
    $build['filter_form'] = \Drupal::formBuilder()->getForm('\Drupal\yamlform\Form\YamlFormSubmissionFilterForm', $this->keys, $this->state, $state_options);

    // Customize.
    if ($this->elements) {
      $route_name = $this->getBaseRouteName() . '.yamlform.results_table.custom';
      $route_parameters = ['yamlform' => $this->yamlform->id()];
      if ($this->sourceEntity) {
        $route_parameters[$this->sourceEntity->getEntityTypeId()] = $this->sourceEntity->id();
      }
      $route_options = ['query' => \Drupal::destination()->getAsArray()];

      $build['custom'] = [
        '#type' => 'link',
        '#title' => $this->t('Customize'),
        '#url' => Url::fromRoute($route_name, $route_parameters, $route_options),
        '#attributes' => [
          'class' => ['yamlform-results-custom', _yamlform_use_ajax('dialog')],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => Json::encode(['width' => 400, 'height' => 600]),
        ],
      ];
    }

    $build += parent::render();

    $build['table']['#attributes']['class'][] = 'yamlform-results';

    $build['#attached']['library'][] = 'yamlform/yamlform';

    return $build;
  }

  /****************************************************************************/
  // Header functions.
  /****************************************************************************/

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $responsive_priorities = [
      'created' => RESPONSIVE_PRIORITY_MEDIUM,
      'langcode' => RESPONSIVE_PRIORITY_LOW,
      'remote_addr' => RESPONSIVE_PRIORITY_LOW,
      'uid' => RESPONSIVE_PRIORITY_MEDIUM,
      'yamlform' => RESPONSIVE_PRIORITY_LOW,
    ];

    $header = [];
    foreach ($this->columns as $column_name => $column_label) {
      $header[$column_name] = $this->buildHeaderColumn($column_name, $column_label);
      if (isset($responsive_priorities[$column_name])) {
        $header[$column_name]['class'][] = $responsive_priorities[$column_name];
      }
    }
    return $header;
  }

  /**
   * Build table header column.
   *
   * @param string $name
   *   Column name.
   * @param $label
   *   Column label.
   *
   * @return array
   *   A renderable array containing a table header column.
   *
   * @throws \Exception
   *   Throw exception if table header column is not found.
   */
  protected function buildHeaderColumn($name, $label) {
    switch ($name) {
      case 'created':
      case 'completed':
      case 'changed':
      case 'langcode':
      case 'remote_addr':
      case 'uid':
      case 'uuid':
        return [
          'data' => $label,
          'field' => $name,
          'specifier' => $name,
        ];

      case 'sid':
        return [
          'data' => $label,
          'field' => 'sid',
          'specifier' => 'sid',
          'sort' => 'desc',
        ];

      case 'entity':
      case 'operations':
      case 'yamlform':
        return ['data' => $label];

      case 'notes':
        return [
          'data' => $this->t('<span class="yamlform-icon yamlform-icon-notes yamlform-icon-notes-on"></span>'),
          'class' => ['yamlform-results-icon'],
        ];

      case 'sticky':
        return [
          'data' => $this->t('<span class="yamlform-icon yamlform-icon-sticky yamlform-icon-sticky-link"></span>'),
          'field' => 'sticky',
          'specifier' => 'sticky',
          'sort' => 'desc',
          'class' => ['yamlform-results-icon'],
        ];

      default:
        if (strpos($name, 'element__') === 0) {
          return $label;
        }
        else {
          throw new \Exception('Column name ' . $name . 'not found.');
        }
    }
  }

  /****************************************************************************/
  // Row functions.
  /****************************************************************************/

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row = [];
    foreach ($this->columns as $column_name => $column_label) {
      $row[$column_name] = $this->buildRowColumn($column_name, $entity);
    }
    return $row;
  }

  /**
   * Build row column
   *
   * @param $name
   *   Column name.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   A YAML form submission.
   *
   * @return array|mixed
   *   The row column value or renderable array.
   *
   * @throws \Exception
   *   Throw exception if table row column is not found.
   */
  public function buildRowColumn($name, EntityInterface $entity) {
    switch ($name) {
      case 'created':
      case 'completed':
      case 'changed':
        return \Drupal::service('date.formatter')->format($entity->created->value);

      case 'entity':
        return ($source_entity = $entity->getSourceEntity()) ? $source_entity->toLink() : '';

      case 'langcode':
        return \Drupal::languageManager()->getLanguage($entity->langcode->value)->getName();

      case 'notes':
        $route_name = $this->getBaseRouteName() . '.yamlform_submission.notes_form';
        $route_parameters = $this->getRouteParameters($entity);
        $route_options = ['query' => \Drupal::destination()->getAsArray()];
        $state = $entity->get('notes')->value ? 'on' : 'off';
        return [
          'data' => [
            '#type' => 'link',
            '#title' => new FormattableMarkup('<span class="yamlform-icon yamlform-icon-notes yamlform-icon-notes-@state"></span>', ['@state' => $state]),
            '#url' => Url::fromRoute($route_name, $route_parameters, $route_options),
            '#attributes' => [
              'class' => [_yamlform_use_ajax('dialog')],
              'data-dialog-type' => 'modal',
              'data-dialog-options' => Json::encode(['width' => 400]),
            ],
          ],
          'class' => ['yamlform-results-icon'],
        ];

      case 'operations':
        return ['data' => $this->buildOperations($entity)];

      case 'remote_addr':
        return $entity->getRemoteAddr();

      case 'sid':
        $route_name = $this->getBaseRouteName() . '.yamlform_submission.canonical';
        $route_parameters = $this->getRouteParameters($entity);
        $link_text = $entity->id() . ($entity->isDraft() ? ' (' . $this->t('draft') . ')' : '');
        return Link::createFromRoute($link_text, $route_name, $route_parameters);

      case 'sticky':
        $route_name = 'entity.yamlform_submission.sticky_toggle';
        $route_parameters = ['yamlform_submission' => $entity->id()];
        $state = $entity->isSticky() ? 'on' : 'off';
        return [
          'data' => [
            '#type' => 'link',
            '#title' => new FormattableMarkup('<span class="yamlform-icon yamlform-icon-sticky yamlform-icon-sticky-@state"></span>', ['@state' => $state]),
            '#url' => Url::fromRoute($route_name, $route_parameters),
            '#attributes' => [
              'id' => 'yamlform-submission-' . $entity->id() . '-sticky',
              'class' => ['use-ajax'],
            ],
          ],
          'class' => ['yamlform-results-icon'],
        ];

      case 'uid':
        return $entity->getOwner()->getAccountName() ?: t('Anonymous');

      case 'uuid':
        return $entity->uuid();

      case 'yamlform':
        return $entity->getYamlForm()->toLink();

      default:
        if (strpos($name, 'element__') === 0) {
          $element_name = preg_replace('/^element__/', '', $name);
          if (isset($this->elements[$element_name])) {
            $element = $this->elements[$element_name];
            $data = $entity->getData();
            $options = [];
            $html = $this->elementManager->invokeMethod('formatHtml', $element, $data[$element_name], $options);
            return (is_array($html)) ? ['data' => $html] : $html;
          }
        }

        throw new \Exception('Column name ' . $name . 'not found.');
    }
  }

  /****************************************************************************/
  // Operations.
  /****************************************************************************/

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $base_route_name = $this->getBaseRouteName();
    $route_parameters = $this->getRouteParameters($entity);
    $route_options = ['query' => \Drupal::destination()->getAsArray()];

    $operations = [];

    if ($entity->access('update')) {
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'weight' => 10,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.edit_form", $route_parameters, $route_options),
      ];
    }

    if ($entity->access('view')) {
      $operations['view'] = [
        'title' => $this->t('View'),
        'weight' => 20,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.canonical", $route_parameters),
      ];
    }

    if ($entity->access('update')) {
      $operations['notes'] = [
        'title' => $this->t('Notes'),
        'weight' => 21,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.notes_form", $route_parameters, $route_options),
      ];
      $operations['resend'] = [
        'title' => $this->t('Resend'),
        'weight' => 22,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.resend_form", $route_parameters, $route_options),
      ];
    }

    if ($entity->access('delete')) {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'weight' => 100,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.delete_form", $route_parameters, $route_options),
      ];
    }

    return $operations;
  }

  /****************************************************************************/
  // Route functions.
  /****************************************************************************/

  /**
   * Get base route name for the YAML form or YAML form source entity.
   *
   * @return string
   *   The base route name for YAML form or YAML form source entity.
   */
  protected function getBaseRouteName() {
    if ($this->sourceEntity) {
      return 'entity.' . $this->sourceEntity->getEntityTypeId();
    }
    else {
      return 'entity';
    }
  }

  /**
   * Get route parameters for the YAML form or YAML form source entity.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   Route parameters for the YAML form or YAML form source entity.
   */
  protected function getRouteParameters(YamlFormSubmissionInterface $yamlform_submission) {
    $route_parameters = ['yamlform_submission' => $yamlform_submission->id()];
    if ($this->sourceEntity) {
      $route_parameters[$this->sourceEntity->getEntityTypeId()] = $this->sourceEntity->id();
    }
    return $route_parameters;
  }

  /****************************************************************************/
  // Query functions.
  /****************************************************************************/

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    $header = $this->buildHeader();
    return $this->getQuery($this->keys, $this->state)
      ->pager($this->limit)
      ->tableSort($header)
      ->execute();
  }

  /**
   * Get the total number of submissions.
   *
   * @param string $keys
   *   (optional) Search key.
   * @param string $state
   *   (optional) Submission state. Can be 'starred' or 'unstarred'.
   *
   * @return int
   *   The total number of submissions.
   */
  protected function getTotal($keys = '', $state = '') {
    return $this->getQuery($keys, $state)
      ->count()
      ->execute();
  }

  /**
   * Get the base entity query filtered by YAML form and search.
   *
   * @param string $keys
   *   (optional) Search key.
   * @param string $state
   *   (optional) Submission state. Can be 'starred' or 'unstarred'.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   An entity query.
   */
  protected function getQuery($keys = '', $state = '') {
    $query = $this->getStorage()->getQuery();
    $this->addQueryConditions($query);

    // Filter by key(word).
    if ($keys) {
      $sub_query = db_select('yamlform_submission_data', 'sd')
        ->fields('sd', ['sid'])
        ->condition('value', '%' . $keys . '%', 'LIKE');
      $this->addQueryConditions($sub_query);

      $or = $query->orConditionGroup()
        ->condition('sid', $sub_query, 'IN')
        ->condition('notes', '%' . $keys . '%', 'LIKE');

      $query->condition($or);
    }

    // Filter by (submission) state.
    if ($state == self::STATE_STARRED || $state == self::STATE_UNSTARRED) {
      $query->condition('sticky', ($state == self::STATE_STARRED) ? 1 : 0);
    }

    return $query;
  }

  /**
   * Add YAML form, account, and source entity conditions to a query.
   *
   * @param \Drupal\Core\Database\Query\AlterableInterface $query
   *   The query to execute.
   */
  protected function addQueryConditions(AlterableInterface $query) {
    // Limit submission to the current YAML form.
    if ($this->yamlform) {
      $query->condition('yamlform_id', $this->yamlform->id());
    }

    // Limit submission to the current user.
    if ($this->account) {
      $query->condition('uid', $this->account->id());
    }

    // Filter entity type and id. (Currently only applies to yamlform_node.module)
    if ($this->sourceEntity) {
      $query->condition('entity_type', $this->sourceEntity->getEntityTypeId());
      $query->condition('entity_id', $this->sourceEntity->id());
    }
  }

}
